"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.BottomSheet = void 0;

var _react = _interopRequireWildcard(require("react"));

var _lodash = _interopRequireDefault(require("lodash.isequal"));

var _invariant = _interopRequireDefault(require("invariant"));

var _reactNativeReanimated = _interopRequireWildcard(require("react-native-reanimated"));

var _reactNativeGestureHandler = require("react-native-gesture-handler");

var _hooks = require("../../hooks");

var _contexts = require("../../contexts");

var _constants = require("../../constants");

var _contentWrapper = _interopRequireDefault(require("../contentWrapper"));

var _draggableView = require("../draggableView");

var _handle = require("../handle");

var _styles = require("./styles");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

_reactNativeReanimated.default.addWhitelistedUIProps({
  decelerationRate: true
});

const BottomSheetComponent = /*#__PURE__*/(0, _react.forwardRef)(({
  // animations
  animationDuration = _constants.DEFAULT_ANIMATION_DURATION,
  animationEasing = _constants.DEFAULT_ANIMATION_EASING,
  // general
  initialSnapIndex = 0,
  snapPoints: _snapPoints,
  topInset = 0,
  onAnimateDistinct = true,
  // animated callback shared values
  animatedPosition: _animatedPosition,
  animatedPositionIndex: _animatedPositionIndex,
  // callbacks
  onChange: _onChange,
  onAnimate,
  // components
  handleComponent: HandleComponent = _handle.BottomSheetHandle,
  backgroundComponent: BackgroundComponent = null,
  children
}, ref) => {
  //#region validate props
  // validate `snapPoints`
  (0, _invariant.default)(_snapPoints, "'snapPoints' was not provided! please provide at least one snap point.");
  (0, _invariant.default)(_snapPoints.length > 0, "'snapPoints' was provided with no points! please provide at least one snap point."); // validate `initialSnapIndex`

  (0, _invariant.default)(typeof initialSnapIndex === 'number', "'initialSnapIndex' was provided but with wrong type ! expected type is a number.");
  (0, _invariant.default)(initialSnapIndex >= -1 && initialSnapIndex <= _snapPoints.length - 1, "'initialSnapIndex' was provided but out of the provided snap points range! expected value to be between -1, ".concat(_snapPoints.length - 1)); // topInset

  (0, _invariant.default)(typeof topInset === 'number', "'topInset' was provided but with wrong type ! expected type is a number."); // validate animations

  (0, _invariant.default)(typeof animationDuration === 'number', "'animationDuration' was provided but with wrong type ! expected type is a number.");
  (0, _invariant.default)(animationDuration > 0, "'animationDuration' was provided but the value is very low! expected value to be greater than 0");
  (0, _invariant.default)(typeof animationEasing === 'function', "'animationEasing' was provided but with wrong type ! expected type is a Animated.EasingFunction."); //#endregion
  //#region variables

  const currentPositionIndex = (0, _reactNativeReanimated.useSharedValue)(initialSnapIndex); // scrollable variables

  const {
    scrollableContentOffsetY,
    scrollableDecelerationRate,
    setScrollableRef,
    removeScrollableRef,
    flashScrollableIndicators
  } = (0, _hooks.useScrollable)();
  const {
    snapPoints,
    sheetHeight,
    onContentLayout,
    isWaitingLayout
  } = (0, _hooks.useSnapPoints)(_snapPoints, topInset);
  const sharedSnapPoints = (0, _reactNativeReanimated.useDerivedValue)(() => {
    return snapPoints;
  }, [snapPoints]); // console.log(
  //   `snapPoints: ${JSON.stringify(
  //     snapPoints
  //   )}, sheetHeight: ${sheetHeight}, isWaitingLayout: ${isWaitingLayout}`
  // );
  // console.log(
  //   `initialSnapIndex: ${initialSnapIndex}, currentPositionIndexRef: ${currentPositionIndexRef.current}`
  // );

  const initialPosition = (0, _react.useMemo)(() => {
    return initialSnapIndex < 0 || isWaitingLayout ? sheetHeight : snapPoints[initialSnapIndex];
  }, [initialSnapIndex, sheetHeight, snapPoints, isWaitingLayout]); // content wrapper

  const contentWrapperGestureRef = (0, _react.useRef)(null);
  const contentWrapperGestureState = (0, _reactNativeReanimated.useSharedValue)(_reactNativeGestureHandler.State.UNDETERMINED);
  const contentWrapperMaxDeltaY = (0, _react.useMemo)(() => snapPoints[Math.max(initialSnapIndex, 0)], [snapPoints, initialSnapIndex]); //#endregion
  //#region private methods

  const refreshUIElements = (0, _react.useCallback)(() => {
    const _currentPositionIndex = Math.max(currentPositionIndex.value, 0);

    if (_currentPositionIndex === snapPoints.length - 1) {
      flashScrollableIndicators(); // @ts-ignore

      contentWrapperGestureRef.current.setNativeProps({
        maxDeltaY: 0
      });
    } else {
      // @ts-ignore
      contentWrapperGestureRef.current.setNativeProps({
        maxDeltaY: snapPoints[_currentPositionIndex]
      });
    }
  }, [currentPositionIndex, snapPoints, flashScrollableIndicators]);
  const handleOnChange = (0, _hooks.useStableCallback)(index => {
    currentPositionIndex.value = index;

    if (_onChange) {
      /**
       * to avoid having -0 🤷‍♂️
       */
      _onChange(index + 1 - 1);
    }
  });
  const handleSettingScrollableRef = (0, _react.useCallback)(scrollableRef => {
    setScrollableRef(scrollableRef);
    refreshUIElements();
  }, [setScrollableRef, refreshUIElements]); //#endregion
  //#region gesture interaction / animation
  // variables

  const animationState = (0, _reactNativeReanimated.useSharedValue)(_constants.ANIMATION_STATE.UNDETERMINED); // console.log(`initialPosition: ${initialPosition}`);

  const animatedPosition = (0, _reactNativeReanimated.useSharedValue)(initialPosition);
  const animatedPositionIndex = (0, _reactNativeReanimated.useDerivedValue)(() => {
    const input = isWaitingLayout ? [sheetHeight, sheetHeight] : [sheetHeight].concat(snapPoints.slice()); // console.log(`input: ${input}, typeof input: ${typeof input}`);

    const output = isWaitingLayout ? [-1, -1] : [-1].concat(snapPoints.slice().map((_, index) => index)); // console.log(`output: ${output}, typeof output: ${typeof output}`);

    const interpolatedIndex = (0, _reactNativeReanimated.interpolate)(animatedPosition.value, input.reverse(), output.reverse(), _reactNativeReanimated.Extrapolate.CLAMP); // console.log(
    //   `animatedPositionIndex: animatedPosition: ${animatedPosition.value}, interpolatedIndex: ${interpolatedIndex}, input: ${input}, output: ${output}`
    // );

    return interpolatedIndex;
  }, [snapPoints, sheetHeight, isWaitingLayout]); // TODO: Uncomment this to watch on animatedPosition
  // useDerivedValue(() => {
  //   console.log(
  //     `animatedPosition: ${animatedPosition.value}, animatedPositionIndex: ${animatedPositionIndex.value}`
  //   );
  //   return animatedPosition.value;
  // }, []);
  // callbacks

  const animateToPointCompleted = (0, _react.useCallback)(isCancelled => {
    'worklet';

    animationState.value = _constants.ANIMATION_STATE.STOPPED;

    if (!isCancelled) {
      return;
    }

    const tempCurrentPositionIndex = Math.round(animatedPositionIndex.value);

    if (tempCurrentPositionIndex !== currentPositionIndex.value) {
      (0, _reactNativeReanimated.runOnJS)(handleOnChange)(tempCurrentPositionIndex);
      (0, _reactNativeReanimated.runOnJS)(refreshUIElements)();
    }
  }, [animatedPositionIndex.value, animationState, handleOnChange, refreshUIElements, currentPositionIndex]);
  const animateToPoint = (0, _react.useCallback)(point => {
    'worklet';

    if (onAnimate !== undefined) {
      const animateFromIndex = currentPositionIndex.value;
      const animateToIndex = sharedSnapPoints.value.findIndex(it => it === point);

      if (!onAnimateDistinct || animateFromIndex !== animateToIndex) {
        (0, _reactNativeReanimated.runOnJS)(onAnimate)(animateFromIndex, animateToIndex);
      }
    }

    animationState.value = _constants.ANIMATION_STATE.RUNNING;
    animatedPosition.value = (0, _reactNativeReanimated.withTiming)(point, {
      duration: animationDuration,
      easing: animationEasing
    }, animateToPointCompleted);
  }, [animationState, animatedPosition, currentPositionIndex, sharedSnapPoints, animationDuration, animationEasing, animateToPointCompleted, onAnimateDistinct, onAnimate]); // hooks

  const [handlePanGestureHandler] = (0, _hooks.useInteractivePanGestureHandler)(_constants.GESTURE.HANDLE, animatedPosition, snapPoints, animateToPoint);
  const [contentPanGestureHandler] = (0, _hooks.useInteractivePanGestureHandler)(_constants.GESTURE.CONTENT, animatedPosition, snapPoints, animateToPoint, scrollableContentOffsetY); //#endregion
  //#region public methods

  const handleSnapTo = (0, _react.useCallback)(index => {
    (0, _invariant.default)(index >= -1 && index <= snapPoints.length - 1, "'index' was provided but out of the provided snap points range! expected value to be between -1, ".concat(snapPoints.length - 1));
    (0, _reactNativeReanimated.runOnUI)(animateToPoint)(snapPoints[index]);
  }, [animateToPoint, snapPoints]);
  const handleClose = (0, _react.useCallback)(() => {
    (0, _reactNativeReanimated.runOnUI)(animateToPoint)(sheetHeight);
  }, [animateToPoint, sheetHeight]);
  const handleExpand = (0, _react.useCallback)(() => {
    (0, _reactNativeReanimated.runOnUI)(animateToPoint)(snapPoints[snapPoints.length - 1]);
  }, [animateToPoint, snapPoints]);
  const handleCollapse = (0, _react.useCallback)(() => {
    (0, _reactNativeReanimated.runOnUI)(animateToPoint)(snapPoints[0]);
  }, [animateToPoint, snapPoints]);
  (0, _react.useImperativeHandle)(ref, () => ({
    snapTo: handleSnapTo,
    expand: handleExpand,
    collapse: handleCollapse,
    close: handleClose
  })); //#endregion
  //#region contexts variables

  const internalContextVariables = (0, _react.useMemo)(() => {
    // console.log(`internalContextVariables is updated!`);
    return {
      animatedPosition,
      animationState,
      contentWrapperGestureRef,
      contentPanGestureHandler,
      scrollableContentOffsetY,
      scrollableDecelerationRate,
      setScrollableRef: handleSettingScrollableRef,
      removeScrollableRef
    };
  }, // eslint-disable-next-line react-hooks/exhaustive-deps
  [contentPanGestureHandler]);
  const externalContextVariables = (0, _react.useMemo)(() => ({
    snapTo: handleSnapTo,
    expand: handleExpand,
    collapse: handleCollapse,
    close: handleClose
  }), [handleSnapTo, handleExpand, handleCollapse, handleClose]); //#endregion
  //#region styles

  const contentContainerStyle = (0, _react.useMemo)(() => ({ ..._styles.styles.container
  }), []);
  const contentContainerAnimatedStyle = (0, _reactNativeReanimated.useAnimatedStyle)(() => {
    return {
      transform: [{
        translateY: animatedPosition.value
      }]
    };
  }, []); //#endregion
  //#region effects

  (0, _reactNativeReanimated.useAnimatedReaction)(() => animatedPosition.value, value => {
    if (_animatedPosition !== undefined) {
      _animatedPosition.value = value;
    }
  }, [_animatedPosition]);
  (0, _reactNativeReanimated.useAnimatedReaction)(() => animatedPositionIndex.value, value => {
    if (_animatedPositionIndex !== undefined) {
      _animatedPositionIndex.value = value;
    }
  }, [_animatedPositionIndex]); //#endregion
  // render
  // console.log(
  //   `[ContentWrapper] maxDeltaY: ${contentWrapperMaxDeltaY}, height: ${sheetHeight}`
  // );

  return /*#__PURE__*/_react.default.createElement(_react.default.Fragment, null, /*#__PURE__*/_react.default.createElement(_contentWrapper.default, {
    ref: contentWrapperGestureRef,
    gestureState: contentWrapperGestureState,
    maxDeltaY: contentWrapperMaxDeltaY
  }, /*#__PURE__*/_react.default.createElement(_reactNativeReanimated.default.View, {
    style: [contentContainerStyle, contentContainerAnimatedStyle]
  }, BackgroundComponent && /*#__PURE__*/_react.default.createElement(BackgroundComponent, {
    pointerEvents: "none"
  }), /*#__PURE__*/_react.default.createElement(_contexts.BottomSheetProvider, {
    value: externalContextVariables
  }, /*#__PURE__*/_react.default.createElement(_reactNativeGestureHandler.PanGestureHandler, {
    simultaneousHandlers: contentWrapperGestureRef,
    shouldCancelWhenOutside: false,
    onGestureEvent: handlePanGestureHandler
  }, /*#__PURE__*/_react.default.createElement(_reactNativeReanimated.default.View, null, /*#__PURE__*/_react.default.createElement(HandleComponent, null))), /*#__PURE__*/_react.default.createElement(_contexts.BottomSheetInternalProvider, {
    value: internalContextVariables
  }, /*#__PURE__*/_react.default.createElement(_draggableView.BottomSheetDraggableView, {
    style: [_styles.styles.contentContainer],
    onLayout: onContentLayout
  }, typeof children === 'function' ? children() : children))))));
});
const BottomSheet = /*#__PURE__*/(0, _react.memo)(BottomSheetComponent, _lodash.default);
exports.BottomSheet = BottomSheet;
//# sourceMappingURL=BottomSheet.js.map