"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useInteractivePanGestureHandler = void 0;

var _reactNativeReanimated = require("react-native-reanimated");

var _reactNativeGestureHandler = require("react-native-gesture-handler");

var _reactNativeRedash = require("react-native-redash");

const useInteractivePanGestureHandler = (type, animatedPosition, snapPoints, animateToPoint, offset) => {
  const gestureState = (0, _reactNativeReanimated.useSharedValue)(_reactNativeGestureHandler.State.UNDETERMINED);
  const gestureTranslationY = (0, _reactNativeReanimated.useSharedValue)(0);
  const gestureVelocityY = (0, _reactNativeReanimated.useSharedValue)(0);
  const animatedSnapPoints = (0, _reactNativeReanimated.useDerivedValue)(() => {
    return snapPoints;
  }, [snapPoints]);
  const gestureHandler = (0, _reactNativeReanimated.useAnimatedGestureHandler)({
    onStart: ({
      state,
      translationY,
      velocityY
    }, context) => {
      // cancel current animation
      (0, _reactNativeReanimated.cancelAnimation)(animatedPosition); // store current animated position

      context.lastAnimatedPosition = animatedPosition.value; // set variables

      gestureState.value = state;
      gestureTranslationY.value = translationY;
      gestureVelocityY.value = velocityY;
    },
    onActive: ({
      state,
      translationY,
      velocityY
    }, context) => {
      gestureState.value = state;
      gestureTranslationY.value = translationY;
      gestureVelocityY.value = velocityY;
      animatedPosition.value = (0, _reactNativeRedash.clamp)(context.lastAnimatedPosition + translationY + (offset && context.lastAnimatedPosition === 0 ? offset.value : 0) * -1, animatedSnapPoints.value[animatedSnapPoints.value.length - 1], animatedSnapPoints.value[0]);
    },
    onEnd: ({
      state
    }, context) => {
      gestureState.value = state;

      if ((offset ? offset.value : 0) > 0 && context.lastAnimatedPosition === 0 && animatedPosition.value === 0) {
        return;
      }

      animateToPoint((0, _reactNativeRedash.snapPoint)(gestureTranslationY.value + context.lastAnimatedPosition, gestureVelocityY.value, animatedSnapPoints.value));
    }
  }, [type, snapPoints, animateToPoint]);
  return [gestureHandler, gestureState, gestureTranslationY, gestureVelocityY];
};

exports.useInteractivePanGestureHandler = useInteractivePanGestureHandler;
//# sourceMappingURL=useInteractivePanGestureHandler.js.map