"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useScrollableInternal = void 0;

var _react = require("react");

var _reactNative = require("react-native");

var _reactNativeReanimated = require("react-native-reanimated");

var _useScrollableAnimatedProps = require("./useScrollableAnimatedProps");

var _useBottomSheetInternal = require("./useBottomSheetInternal");

const useScrollableInternal = type => {
  // refs
  const scrollableRef = (0, _reactNativeReanimated.useAnimatedRef)();
  const scrollablePosition = (0, _reactNativeReanimated.useSharedValue)(0);
  const scrollableContentOffsetY = (0, _reactNativeReanimated.useSharedValue)(0); // hooks

  const scrollableAnimatedProps = (0, _useScrollableAnimatedProps.useScrollableAnimatedProps)();
  const {
    animatedPosition,
    scrollableContentOffsetY: _scrollableContentOffsetY,
    setScrollableRef,
    removeScrollableRef
  } = (0, _useBottomSheetInternal.useBottomSheetInternal)(); // callbacks

  const handleScrollEvent = (0, _reactNativeReanimated.useAnimatedScrollHandler)({
    onBeginDrag: ({
      contentOffset: {
        y
      }
    }) => {
      if (animatedPosition.value !== 0) {
        return;
      }

      scrollablePosition.value = y;
      scrollableContentOffsetY.value = y;
      _scrollableContentOffsetY.value = y;
    },
    onScroll: ({
      contentOffset: {
        y
      }
    }) => {
      if (animatedPosition.value !== 0) {
        // @ts-ignore
        (0, _reactNativeReanimated.scrollTo)(scrollableRef, 0, scrollablePosition.value, false);
        return;
      }

      scrollablePosition.value = y;
    },
    onEndDrag: () => {
      if (animatedPosition.value !== 0) {
        // @ts-ignore
        (0, _reactNativeReanimated.scrollTo)(scrollableRef, 0, scrollablePosition.value, false);
        return;
      }
    },
    onMomentumEnd: () => {
      if (animatedPosition.value !== 0) {
        // @ts-ignore
        (0, _reactNativeReanimated.scrollTo)(scrollableRef, 0, scrollablePosition.value, false);
        return;
      }
    }
  }, []);
  const handleSettingScrollable = (0, _react.useCallback)(() => {
    // set current content offset
    (0, _reactNativeReanimated.runOnUI)(() => {
      _scrollableContentOffsetY.value = scrollableContentOffsetY.value;
    })(); // set current scrollable ref

    const id = (0, _reactNative.findNodeHandle)(scrollableRef.current);

    if (id) {
      setScrollableRef({
        id: id,
        type,
        node: scrollableRef,
        didResize: false
      });
    } else {
      console.warn("Couldn't find the scrollable node handle id!");
    }

    return () => {
      removeScrollableRef(scrollableRef);
    }; // eslint-disable-next-line react-hooks/exhaustive-deps
  }, []);
  return {
    scrollableRef,
    scrollableAnimatedProps,
    handleScrollEvent,
    handleSettingScrollable
  };
};

exports.useScrollableInternal = useScrollableInternal;
//# sourceMappingURL=useScrollableInternal.js.map