"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useSnapPoints = void 0;

var _reactNative = require("react-native");

var _react = require("react");

var _remeda = require("remeda");

/**
 * Converts snap points with percentage to fixed numbers.
 */
const useSnapPoints = (snapPoints, topInset, handleHeight = 24) => {
  const {
    height: windowHeight
  } = (0, _reactNative.useWindowDimensions)();
  const [contentHeight, setContentHeight] = (0, _react.useState)(undefined);
  const onContentLayout = (0, _react.useCallback)(({
    nativeEvent: {
      layout: {
        height
      }
    }
  }) => {
    // console.log(`onContentLayout: height: ${height}`);
    setContentHeight(height);
  }, []);
  const hasContentSnapPoints = (0, _react.useMemo)(() => {
    return snapPoints.some(it => it.relativeTo === 'content');
  }, [snapPoints]);
  const maxPossibleSnapPoint = (0, _react.useMemo)(() => Math.max(windowHeight - topInset, 0), [windowHeight, topInset]);
  const snapOffsets = (0, _react.useMemo)(() => {
    if (hasContentSnapPoints && contentHeight === undefined) {
      return undefined;
    } else {
      const snapPointsOffsets = snapPoints.map(it => {
        let position;

        if (it.relativeTo === 'window') {
          position = it.percentagesOf / 100 * (windowHeight - topInset);
        } else {
          position = it.percentagesOf / 100 * contentHeight + handleHeight;
        }

        return (0, _remeda.clamp)(position, {
          min: 0,
          max: maxPossibleSnapPoint
        });
      }).sort((first, second) => first === second ? 0 : first < second ? -1 : 1);
      return snapPointsOffsets;
    }
  }, [snapPoints, maxPossibleSnapPoint, topInset, windowHeight, contentHeight, handleHeight, hasContentSnapPoints]);
  const {
    adjustedSnapPoints,
    sheetHeight,
    maxSnapPoint
  } = (0, _react.useMemo)(() => {
    if (snapOffsets !== undefined) {
      const maxSnapPoint = snapOffsets[snapOffsets.length - 1];
      return {
        adjustedSnapPoints: snapOffsets.map(it => maxSnapPoint - it),
        maxSnapPoint: maxSnapPoint,
        sheetHeight: maxSnapPoint
      };
    } else {
      return {
        adjustedSnapPoints: [windowHeight],
        maxSnapPoint: windowHeight,
        sheetHeight: windowHeight
      };
    }
  }, [snapOffsets, windowHeight]); // console.log(
  //   `windowHeight: ${windowHeight}, contentHeight: ${contentHeight}, handleHeight: ${handleHeight}, positions: ${JSON.stringify(
  //     snapOffsets
  //   )}, snapPoints: ${JSON.stringify(snapPoints)}, topInset: ${topInset}`
  // );

  return {
    snapPoints: adjustedSnapPoints,
    maxSnapPoint: maxSnapPoint,
    sheetHeight: sheetHeight,
    onContentLayout: onContentLayout,
    isWaitingLayout: snapOffsets === undefined
  };
};

exports.useSnapPoints = useSnapPoints;
//# sourceMappingURL=useSnapPoints.js.map