import { useAnimatedGestureHandler, useSharedValue, cancelAnimation, useDerivedValue } from 'react-native-reanimated';
import { State } from 'react-native-gesture-handler';
import { clamp, snapPoint } from 'react-native-redash';
export const useInteractivePanGestureHandler = (type, animatedPosition, snapPoints, animateToPoint, offset) => {
  const gestureState = useSharedValue(State.UNDETERMINED);
  const gestureTranslationY = useSharedValue(0);
  const gestureVelocityY = useSharedValue(0);
  const animatedSnapPoints = useDerivedValue(() => {
    return snapPoints;
  }, [snapPoints]);
  const gestureHandler = useAnimatedGestureHandler({
    onStart: ({
      state,
      translationY,
      velocityY
    }, context) => {
      // cancel current animation
      cancelAnimation(animatedPosition); // store current animated position

      context.lastAnimatedPosition = animatedPosition.value; // set variables

      gestureState.value = state;
      gestureTranslationY.value = translationY;
      gestureVelocityY.value = velocityY;
    },
    onActive: ({
      state,
      translationY,
      velocityY
    }, context) => {
      gestureState.value = state;
      gestureTranslationY.value = translationY;
      gestureVelocityY.value = velocityY;
      animatedPosition.value = clamp(context.lastAnimatedPosition + translationY + (offset && context.lastAnimatedPosition === 0 ? offset.value : 0) * -1, animatedSnapPoints.value[animatedSnapPoints.value.length - 1], animatedSnapPoints.value[0]);
    },
    onEnd: ({
      state
    }, context) => {
      gestureState.value = state;

      if ((offset ? offset.value : 0) > 0 && context.lastAnimatedPosition === 0 && animatedPosition.value === 0) {
        return;
      }

      animateToPoint(snapPoint(gestureTranslationY.value + context.lastAnimatedPosition, gestureVelocityY.value, animatedSnapPoints.value));
    }
  }, [type, snapPoints, animateToPoint]);
  return [gestureHandler, gestureState, gestureTranslationY, gestureVelocityY];
};
//# sourceMappingURL=useInteractivePanGestureHandler.js.map